#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/resource.h>

#include <netinet/in.h>
#include <netinet/tcp.h>

#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <netdb.h>
#include <string.h>
#include <stdlib.h>

#include "network.h"
#include "fdevent.h"
#include "log.h"
#include "file_cache.h"

#include "network_backends.h"

#ifdef USE_FREEBSD_SENDFILE

int network_write_chunkqueue_freebsdsendfile(server *srv, connection *con, chunkqueue *cq) {
	const int fd = con->fd;
	chunk *c;
	size_t chunks_written = 0;
	
	for(c = cq->first; c; c = c->next, chunks_written++) {
		int chunk_finished = 0;
		
		switch(c->type) {
		case MEM_CHUNK: {
			char * offset;
			size_t toSend;
			ssize_t r;
			
			size_t num_chunks, i;
			struct iovec chunks[UIO_MAXIOV];
			chunk *tc;
			
			/* build writev list */
			for(num_chunks = 0, tc = c; tc && tc->type == MEM_CHUNK && num_chunks < UIO_MAXIOV; num_chunks++, tc = tc->next);
			
			for(tc = c, i = 0; i < num_chunks; tc = tc->next, i++) {
				offset = tc->b->ptr + tc->written;
				toSend = tc->b->used - 1 - tc->written;
				
				chunks[i].iov_base = offset;
				chunks[i].iov_len = toSend;
			}
			
			if ((r = writev(fd, chunks, num_chunks)) < 0) {
				log_error_write(srv, __FILE__, __LINE__, "ssd", 
						"write failed: ", strerror(errno), fd);
				
				return -1;
			}
			
			/* check which chunks have been written */
			
			for(i = 0, tc = c; i < num_chunks; i++, tc = tc->next) {
				if (r >= chunks[i].iov_len) {
					/* written */
					r -= chunks[i].iov_len;
					tc->written += chunks[i].iov_len;
					
					if (chunk_finished) {
						/* skip the chunks from further touches */
						chunks_written++;
						c = c->next;
					} else {
						/* chunks_written + c = c->next is done in the for()*/
						chunk_finished++;
					}
				} else {
					/* partially written */
					
					tc->written += r;
					chunk_finished = 0;
					
					log_error_write(srv, __FILE__, __LINE__, "sdd", 
						"partially write: ", r, fd);
					
					break;
				}
			}
			
			break;
		}
		case FILE_CHUNK: {
			off_t offset, r;
			size_t toSend;
			
			if (NULL == (con->fce = file_cache_get_entry(srv, con, c->b, con->fce))) {
				log_error_write(srv, __FILE__, __LINE__, "sb",
						strerror(errno), c->b);
				return -1;
			}
			
			offset = c->offset + c->written;
			toSend = c->length - c->written;
			
			/* FreeBSD sendfile() */
			if (-1 == sendfile(con->fce->fd, fd, offset, toSend, NULL, &r, 0)) {
				if (errno != EAGAIN) {
					log_error_write(srv, __FILE__, __LINE__, "ssd", "sendfile: ", strerror(errno), errno);
					
					return -1;
				}
			}

			c->written += r;
			
			if (c->written == c->length) {
				chunk_finished = 1;
			}
			
			break;
		}
		default:
			
			log_error_write(srv, __FILE__, __LINE__, "ds", c, "type not known");
			
			return -1;
		}
		
		if (!chunk_finished) {
			/* not finished yet */
			
			break;
		}
	}

	return chunks_written;
}

#endif
